// Stepper.cpp

#include <graph.h>
#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "Stepper.h"

#define PI				3.1415926
#define DEG2RAD(x)	(((x)*PI)/180.0)
#define STEPS		4

// Stepper

const int s_size = 150 ;
const int s_box_arc = (int)( s_size / 5.0 + 0.5 ) ;
const int s_border = (int)( s_size / 8.0 + 0.5 ) ;
const int s_tic = (int)( s_size / 40.0 + 0.5 ) ;
const int s_ptr = (int)( s_size / 20.0 + 0.5 )  ;
const int s_arc_off = (int)( s_size / 50.0 + 0.5 ) ;
const int s_text = (int)( s_size / 40.0 + 0.5 ) ;
const int s_bwide = (int)( s_size / 6.0 + 0.5 ) ;

// 6 bits to control two bipolar coils using L293
// Forward Spin -->
// D6 CE1      1  0  1  0
// D5 INP1     0  0  1  1
// D4 INP2     1  1  0  0
// D3 CE2      0  1  0  1
// D2 INP3     0  0  1  1
// D1 INP4     1  1  0  0
// steps        29 0d 32 16
// Reverse Spin <--

unsigned char steps [ STEPS ] = { 0x29, 0x0d, 0x32, 0x16 } ;

Stepper::Stepper (  Port *port, int x, int y, char *title, int PortMsb )
{
	s_port = port ;
	s_msb = PortMsb ;
	s_x = x ;
	s_y = y ;
	s_title = new char [ strlen ( title ) + 1 ] ;
	strcpy ( s_title, title ) ;
	SpeedTimer = 0 ;

	spin = 0 ;
	rev = 0 ;
	step_inx = 0 ;
	speed = 1 ;
	byte = 1 ;

	s_cen_x = s_x + ( s_size / 2 ) ;
	s_cen_y = s_y + s_size / 2 ;
	s_rad = (int)( ( s_size - 2 * s_box_arc ) / 2.0 + 0.5 ) ;

	ActPosition = 0 ;
	ShowPosition() ;
	s_port -> OutPort ( s_msb, 8,  steps [ ActPosition % STEPS ] ) ;
}

Stepper::~Stepper()
{
}

void Stepper::SetPosition ( int pos )
{
	if ( pos == ActPosition )
		return ;

	int diff = pos - ActPosition ;
	if ( diff < 0 )
		diff += 100 ;

	if ( diff <= 50 )
	{
		while (  pos != ActPosition )
			FwdStep() ;
	}
	else
	{
		while ( pos != ActPosition )
			RevStep() ;
	}
	ShowPosition() ;
}

void Stepper::ShowPosition()
{
	double x1, y1, theta ;
	
	_setcolor ( 0 ) ;
	theta = ( PrevPosition / 100.0 ) * 360 - 90 ;
	x1 = ( s_rad - 1 ) * cos ( DEG2RAD ( theta ) ) ;
	y1 = ( s_rad - 1 ) * sin ( DEG2RAD ( theta ) ) ;
	_pie_w ( _GFILLINTERIOR, 
		x1 + s_cen_x - s_ptr, y1 + s_cen_y - s_ptr, 
		x1 + s_cen_x + s_ptr, y1 + s_cen_y + s_ptr, 
		x1 + s_cen_x + s_ptr * cos ( DEG2RAD ( theta + 210 ) ),
		y1 + s_cen_y + s_ptr * sin ( DEG2RAD ( theta + 210 ) ),
		x1 + s_cen_x + s_ptr * cos ( DEG2RAD ( theta + 150 ) ),
		y1 + s_cen_y + s_ptr * sin ( DEG2RAD ( theta + 150 ) ) ) ;

	_setcolor ( 12 ) ;
	theta = ( ActPosition / 100.0 ) * 360 - 90 ;
	x1 = ( s_rad - 1 ) * cos ( DEG2RAD ( theta ) ) ;
	y1 = ( s_rad - 1 ) * sin ( DEG2RAD ( theta ) ) ;
	_pie_w ( _GFILLINTERIOR, 
		x1 + s_cen_x - s_ptr, y1 + s_cen_y - s_ptr, 
		x1 + s_cen_x + s_ptr, y1 + s_cen_y + s_ptr, 
		x1 + s_cen_x + s_ptr * cos ( DEG2RAD ( theta + 210 ) ),
		y1 + s_cen_y + s_ptr * sin ( DEG2RAD ( theta + 210 ) ),
		x1 + s_cen_x + s_ptr * cos ( DEG2RAD ( theta + 150 ) ),
		y1 + s_cen_y + s_ptr * sin ( DEG2RAD ( theta + 150 ) ) ) ;
}

void Stepper::DrawPanel()
{
	struct _fontinfo fi ;

	_getfontinfo ( &fi ) ;
	_setcolor ( 7 ) ;
	_rectangle_w ( _GBORDER, s_x, s_y, s_x + s_size, s_y + s_size ) ;
	_moveto ( s_x, s_y + s_border ) ;
	_lineto ( s_x + s_size, s_y + s_border ) ;
	_moveto ( s_x, s_y + s_size - s_border ) ;
	_lineto ( s_x + s_size, s_y + s_size - s_border ) ;
	_moveto_w ( s_x + ( s_size - strlen ( s_title ) * fi . pixwidth ) / 2, 
	 +  s_y + ( s_border - fi . pixheight ) / 2 ) ;
	_outgtext ( s_title ) ;

	_ellipse ( _GBORDER, s_cen_x - s_rad, s_cen_y - s_rad, 
		s_cen_x + s_rad, s_cen_y + s_rad ) ;

	_moveto_w ( s_cen_x + s_rad, s_cen_y ) ;
	_lineto_w ( s_cen_x + s_rad + s_tic, s_cen_y ) ;
	_moveto_w ( s_cen_x - s_rad, s_cen_y ) ;
	_lineto_w ( s_cen_x - s_rad - s_tic, s_cen_y ) ;
	_moveto_w ( s_cen_x, s_cen_y + s_rad ) ;
	_lineto_w ( s_cen_x, s_cen_y + s_rad + s_tic ) ;
	_moveto_w ( s_cen_x, s_cen_y - s_rad ) ;
	_lineto_w ( s_cen_x, s_cen_y - s_rad - s_tic ) ;
	
	int b_x ;
	b_x = s_x ;
	int bwide = (int)( ( s_size / 5.0 ) + 0.5 ) ;
	int i ;
	for ( i = 0 ; i < 5 ; i++ )
	{
		b_x += bwide ;
		_moveto ( b_x, s_y + s_size - s_border ) ;
		_lineto ( b_x, s_y + s_size ) ;
	}
	DrawButton ( s_x, s_y + s_size - s_border, bwide, s_border, B_REW, 1 ) ;
	DrawButton ( s_x, s_y + s_size - s_border, bwide, s_border, B_REV, 1 ) ;
	DrawButton ( s_x, s_y + s_size - s_border, bwide, s_border, B_STOP, 1 ) ;
	DrawButton ( s_x, s_y + s_size - s_border, bwide, s_border, B_FWD, 1 ) ;
	DrawButton ( s_x, s_y + s_size - s_border, bwide, s_border, B_FFWD, 1 ) ;
}

void Stepper::DrawButton ( int x, int y, int w, int h, B_MODES mode, int enable )
{
	int x1, y1, y2, y_c, dx, dy ;
	
	_setcolor ( enable ? 7 : 8 ) ;
	dx = (int)( w / 8.0 + 0.5 ) ;
	dy = (int)( h / 4.0 + 0.5 ) ;
	y1 = y + dy ;
	y2 = y + h - dy ;
	y_c = (int)( y + ( h / 2 + 0.5 ) ) ;
	switch ( mode )
	{
		case B_REW :
			{
				x1 = x + w * 4 ;
				double points [ 3 ] [ 2 ] ;
				points [ 0 ] [ 0 ] = x1 + dx * 2 ;
				points [ 0 ] [ 1 ] = y1 ;
				points [ 1 ] [ 0 ] = x1 + dx * 4 ;
				points [ 1 ] [ 1 ] = y_c ;
				points [ 2 ] [ 0 ] = x1 + dx * 2 ;
				points [ 2 ] [ 1 ] = y2 ;
				_polygon_w ( _GFILLINTERIOR, (double *)points, 3 ) ;
				points [ 0 ] [ 0 ] += dx * 2 ;
				points [ 1 ] [ 0 ] += dx * 2 ;
				points [ 2 ] [ 0 ] += dx * 2 ;
				_polygon_w ( _GFILLINTERIOR, (double *)points, 3 ) ;
			}
			break ;
		case B_REV :
			{
				x1 = x + w * 3 ;
				double points [ 3 ] [ 2 ] ;
				points [ 0 ] [ 0 ] = x1 + dx * 3 ;
				points [ 0 ] [ 1 ] = y1 ;
				points [ 1 ] [ 0 ] = x1 + dx * 5 ;
				points [ 1 ] [ 1 ] = y_c ;
				points [ 2 ] [ 0 ] = x1 + dx * 3 ;
				points [ 2 ] [ 1 ] = y2 ;
				_polygon_w ( _GFILLINTERIOR, (double *)points, 3 ) ;
			}
			break ;
		case B_STOP :
		{
			x1 = x + w * 2 ;
			_ellipse ( _GFILLINTERIOR, x1 + dx * 3, y_c - dx,
				x1 + dx * 5, y_c + dx ) ;
			break ;
		}
		case B_FWD :
			{
				x1 = x + w * 1 ;
				double points [ 3 ] [ 2 ] ;
				points [ 0 ] [ 0 ] = x1 + dx * 5 ;
				points [ 0 ] [ 1 ] = y1 ;
				points [ 1 ] [ 0 ] = x1 + dx * 3 ;
				points [ 1 ] [ 1 ] = y_c ;
				points [ 2 ] [ 0 ] = x1 + dx * 5 ;
				points [ 2 ] [ 1 ] = y2 ;
				_polygon_w ( _GFILLINTERIOR, (double *)points, 3 ) ;
			}
			break ;
		case B_FFWD :
			{
				x1 = x ;
				double points [ 3 ] [ 2 ] ;
				points [ 0 ] [ 0 ] = x1 + dx * 4 ;
				points [ 0 ] [ 1 ] = y1 ;
				points [ 1 ] [ 0 ] = x1 + dx * 2 ;
				points [ 1 ] [ 1 ] = y_c ;
				points [ 2 ] [ 0 ] = x1 + dx * 4 ;
				points [ 2 ] [ 1 ] = y2 ;
				_polygon_w ( _GFILLINTERIOR, (double *)points, 3 ) ;
				points [ 0 ] [ 0 ] += dx * 2 ;
				points [ 1 ] [ 0 ] += dx * 2 ;
				points [ 2 ] [ 0 ] += dx * 2 ;
				_polygon_w ( _GFILLINTERIOR, (double *)points, 3 ) ;
			}
			break ;
	}
}

void Stepper::LClick ( int x, int y )
{
	int bwide = (int)( ( s_size / 5.0 ) + 0.5 ) ;
	
	if ( y > s_y + s_border && y < s_y + s_size - s_border )
	{
		x -= s_cen_x ;
		y -= s_cen_y ;
		double theta = atan2 ( y, x ) ;
		theta *= 180 / PI ;
		theta += 90 ;
		if ( theta < 0 )
			theta += 360 ;
		int pos = (int)( theta / 3.6 ) ;
		SetPosition ( pos ) ;
	}
	else if ( y >= s_y + s_size - s_border && y <= s_y + s_size )
	{
		x -= s_x ;
		switch ( x / bwide )
		{
			case 0 : // REV SPIN
				ExecCmd ( B_REW ) ;
				break ;
			case 1 : // REV STEP
				ExecCmd ( B_REV ) ;
				break ;
			case 2 : // STOP
				ExecCmd ( B_STOP ) ;
				break ;
			case 3 : // FWD STEP
				ExecCmd ( B_FWD ) ;
				break ;
			case 4 : // FWD SPIN
				ExecCmd ( B_FFWD ) ;
				break ;
		}
	}
}

void Stepper::KeyPress ( int keycode )
{
	switch ( keycode )
	{
		case 0xf3 : // REV SPIN
			ExecCmd ( B_REW ) ;
			break ;
		case 0xcb : // REV STEP
			ExecCmd ( B_REV ) ;
			break ;
		case ' ' : // Space - STOP
			ExecCmd ( B_STOP ) ;
			break ;
		case 0xcd : // FWD STEP
			ExecCmd ( B_FWD ) ;
			break ;
		case 0xf4 : // FWD SPIN
			ExecCmd ( B_FFWD ) ;
			break ;
	}
}

void Stepper::ExecCmd ( B_MODES cmd )
{
	switch ( cmd )
	{
		case B_REW : // REV SPIN
			spin = 1 ;
			rev = 1 ;
			SpeedTimer = speed ;
			break ;
		case B_REV : // REV STEP
			RevStep() ;
			break ;
		case B_STOP : // STOP
			spin = 0 ;
			break ;
		case B_FWD : // FWD STEP
			FwdStep() ;
			break ;
		case B_FFWD : // FWD SPIN
			spin = 1 ;
			rev = 0 ;
			SpeedTimer = speed ;
			break ;
	}
}

void Stepper::FwdStep()
{
	PrevPosition = ActPosition ;
	if ( ++ActPosition == 100 )
		ActPosition = 0 ;
	s_port -> OutPort ( s_msb, 8,  steps [ ActPosition % STEPS ] ) ;
	ShowPosition() ;
}

void Stepper::RevStep()
{
	PrevPosition = ActPosition ;
	if ( --ActPosition < 0 )
		ActPosition = 99 ;
	s_port -> OutPort ( s_msb, 8,  steps [ ActPosition % STEPS ] ) ;
	ShowPosition() ;
}

void Stepper::TimerTick()
{
	if ( spin )
	{
		if ( --SpeedTimer == 0 )
		{
			rev ? RevStep() : FwdStep() ;
			SpeedTimer = speed ;
		}
	}
}
